#ifndef MESH_H
#define MESH_H

#include <cstddef>
#include <string>
#include <vector>
#include <cstdint>
#include <QString>
#include "vec.h"
#include "matrix.h"

class QGLShaderProgram;

class Mesh
{
	friend class Parser3DS;
public:
	Mesh();
	~Mesh();

	void clear();

	void draw();

	void load(const QString &filename);
	void save(const QString &filename) const;

	std::pair<Vec3, Vec3> getBoundingBox() const;

	void optimize();

	void swap(Mesh *mesh);

protected:
	void recompute_normals();
	void loadOFF(const QString &filename);
	void loadOBJ(const QString &filename);
	void loadSTL(const QString &filename);
	void load3DS(const QString &filename);
	void loadPLY(const QString &filename);

	void saveOFF(const QString &filename) const;
	void saveSTL(const QString &filename) const;
	void savePLY(const QString &filename) const;

protected:
	std::string name;
	std::vector<Vec3> vertex;
	std::vector<Vec3> normal;
	std::vector<Vec3ub> color;
	std::vector<Vec2> tcoord;
	std::vector<u_int32_t> index;
	u_int32_t primitive_type;
	u_int32_t texture_handle;
	u_int32_t normal_texture_handle;
	std::vector<Mesh*> childs;

	u_int32_t vbo_vtx;
	u_int32_t vbo_normal;
	u_int32_t vbo_color;
	u_int32_t vbo_tcoord;
	u_int32_t vbo_index;

	Mat44 local;

	Vec4 diffuse;
	Vec4 specular;
	float shininess;

private:
	static QGLShaderProgram *pShader;
};

#endif // MESH_H
